#last edited dec1115
#Brian Campbell-Deem cmpblld2
#Physible pt. 3
#MATH198
#MODIFIED CODE. SOURCE: BRUCE SHERWOOD (SEE COMMENTS BELOW)
#MODIFICATIONS MARKED BY DOUBLE COMMENTS (##)

from __future__ import print_function, division
from visual import *
from visual.graph import *
#from visual.graph import *

# Double pendulum

# The analysis is in terms of Lagrangian mechanics.
# The Lagrangian variables are angle of upper bar and angle of lower bar,
# measured from the vertical.

# Bruce Sherwood
#
# Corrections to the Lagrangian calculations by Owen Long, UC. Riverside
#

scene.title = 'Double Pendulum'
scene.height = scene.width = 500

def get_right_value(num):                               ##error handlers
    while(1):
        try:
            value = float(raw_input(num))
        except ValueError:
            continue

        if(value>10 or value<0.1):
            continue
        else:
            break
    return value


g = 9.8

M1 = get_right_value("Please enter the value of the first masses (0.1-10): ")
M2 = get_right_value("Please enter the value of the second masses (0.1-10): ")
flag = "init"

while( (flag!="y") and (flag!="n") and (flag!="yes") and (flag!="no") ):     ##object trails enabled
        flag = raw_input("Would you like to enable object trails? (y/n): ")

if( (flag=="y") or (flag=="yes") ): ##
    trail_flag=True
else:
    trail_flag=False

angular_offset = get_right_value("Choose a small angular offset for the second double pendulum (0.1-10): ") ## offset

L1 = 0.5 # physical length of upper assembly; distance between axles
L2 = 1.0 # physical length of lower bar
I1 = M1*L1**2/12 # moment of inertia of upper assembly
I2 = M2*L2**2/12 # moment of inertia of lower bar
d = 0.05 # thickness of each bar
gap = 2*d # distance between two parts of upper, U-shaped assembly
L1display = L1+d # show upper assembly a bit longer than physical, to overlap axle
L2display = L2+d/2 # show lower bar a bit longer than physical, to overlap axle

hpedestal = 1.3*(L1+L2) # height of pedestal
wpedestal = 0.1 # width of pedestal
tbase = 0.05 # thickness of base
wbase = 8*gap # width of base
offset = 2*gap # from center of pedestal to center of U-shaped upper assembly
top = vector(0,0,0) # top of inner bar of U-shaped upper assembly
scene.center = top-vector(0,(L1+L2)/2.,0)

theta1 = 2.1 # initial upper angle (from vertical) 2.1
theta2 = 2.4 # initial lower angle (from vertical) 2.4
theta1dot = 0 # initial rate of change of theta1
theta2dot = 0 # initial rate of change of theta2

pedestal = box(pos=top-vector(0,hpedestal/2.,offset),
                 height=1.1*hpedestal, length=wpedestal, width=wpedestal,
                 color=(0.4,0.4,0.5))
base = box(pos=top-vector(0,hpedestal+tbase/2.,offset),
                 height=tbase, length=wbase, width=wbase,
                 color=pedestal.color)
axle = cylinder(pos=top-vector(0,0,gap/2.-d/4.), axis=(0,0,-offset), radius=d/4., color=color.yellow)

frame1 = frame(pos=top)
bar1 = box(frame=frame1, pos=(L1display/2.-d/2.,0,-(gap+d)/2.), size=(L1display,d,d), color=color.red)
bar1b = box(frame=frame1, pos=(L1display/2.-d/2.,0,(gap+d)/2.), size=(L1display,d,d), color=color.red)
axle1 = cylinder(frame=frame1, pos=(L1,0,-(gap+d)/2.), axis=(0,0,gap+d),
                 radius=axle.radius, color=axle.color)
frame1.axis = (0,-1,0)
frame2 = frame(pos=frame1.axis*L1)
bar2 = box(frame=frame2, pos=(L2display/2.-d/2.,0,0), size=(L2display,d,d), color=color.green)

ball1 = sphere(frame=frame2, pos=(L2display - d,0,0), radius=L2/20.0, color=color.green)
ball2 = sphere(pos=(L2display - d,0,0), radius=L2/20.0, color=color.green, make_trail=trail_flag)

frame2.axis = (0,-1,0)
frame1.rotate(axis=(0,0,1), angle=theta1)
frame2.rotate(axis=(0,0,1), angle=theta2)
frame2.pos = top+frame1.axis*L1

dt = 0.0001 # energy constancy check fails for dt > 0.0003
t = 0.
t2=0

C11 = (0.25*M1+M2)*L1**2+I1
C22 = 0.25*M2*L2**2+I2

#### For energy check:
##gdisplay(x=800)
##gK = gcurve(color=color.yellow)
##gU = gcurve(color=color.cyan)
##gE = gcurve(color=color.red)

##################################################################################################
##----------SECOND WINDOW------------##

scene2=display(x=scene.width)
scene2.title = 'Double Pendulum'
scene2.height = scene2.width = 500

L3 = 0.5 # physical length of upper assembly; distance between axles
L4 = 1.0 # physical length of lower bar
I3 = M1*L1**2/12 # moment of inertia of upper assembly
I4 = M2*L2**2/12 # moment of inertia of lower bar
d2 = 0.05 # thickness of each bar
gap2 = 2*d # distance between two parts of upper, U-shaped assembly
L3display = L3+d # show upper assembly a bit longer than physical, to overlap axle
L4display = L4+d/2 # show lower bar a bit longer than physical, to overlap axle

hpedestal2 = 1.3*(L3+L4) # height of pedestal
wpedestal2 = 0.1 # width of pedestal
tbase2 = 0.05 # thickness of base
wbase2 = 8*gap2 # width of base
offset2 = 2*gap2 # from center of pedestal to center of U-shaped upper assembly
top2 = vector(0,0,0) # top of inner bar of U-shaped upper assembly
scene2.center = top2-vector(0,(L3+L4)/2.,0)

theta3 = 2.1 # initial upper angle (from vertical) 2.1
theta4 = 2.4 + angular_offset*pi/180 # initial lower angle (from vertical) 2.4
theta3dot = 0 # initial rate of change of theta1
theta4dot = 0 # initial rate of change of theta2

pedestal2 = box(pos=top2-vector(0,hpedestal2/2.,offset),
                                 height=1.1*hpedestal2, length=wpedestal2, width=wpedestal2,
                                 color=(0.4,0.4,0.5))
base2 = box(pos=top2-vector(0,hpedestal2+tbase/2.,offset),
                                 height=tbase2, length=wbase2, width=wbase2,
                                 color=pedestal2.color)
axle2 = cylinder(pos=top2-vector(0,0,gap2/2.-d2/4.), axis=(0,0,-offset), radius=d2/4., color=color.yellow)

frame3 = frame(pos=top2)
bar3 = box(frame=frame3, pos=(L3display/2.-d2/2.,0,-(gap2+d2)/2.), size=(L3display,d2,d2), color=color.red)
bar3b = box(frame=frame3, pos=(L3display/2.-d2/2.,0,(gap2+d2)/2.), size=(L3display,d2,d2), color=color.red)
axle3 = cylinder(frame=frame3, pos=(L3,0,-(gap2+d2)/2.), axis=(0,0,gap2+d2),
                                 radius=axle2.radius, color=axle2.color)
frame3.axis = (0,-1,0)
frame4 = frame(pos=frame3.axis*L3)
bar4 = box(frame=frame4, pos=(L4display/2.-d2/2.,0,0), size=(L4display,d2,d2), color=color.green)

ball3 = sphere(frame=frame4, pos=(L4display - d2,0,0), radius=L4/20.0, color=color.green)
ball4 = sphere(pos=(L4display - d2,0,0), radius=L4/20.0, color=color.green, make_trail=trail_flag)

frame4.axis = (0,-1,0)
frame3.rotate(axis=(0,0,1), angle=theta3)
frame4.rotate(axis=(0,0,1), angle=theta4)
frame4.pos = top2+frame3.axis*L3

C33 = (0.25*M1+M2)*L3**2+I3
C44 = 0.25*M2*L4**2+I4
#############################################################################################################

while True:
    rate(1/dt)
    # Calculate accelerations of the Lagrangian coordinates:

    C12 = C21 = 0.5*M2*L1*L2*cos(theta1-theta2)
    Cdet = C11*C22-C12*C21
    a = .5*M2*L1*L2*sin(theta1-theta2)
    A = -(.5*M1+M2)*g*L1*sin(theta1)-a*theta2dot**2
    B = -.5*M2*g*L2*sin(theta2)+a*theta1dot**2
    atheta1 = (C22*A-C12*B)/Cdet
    atheta2 = (-C21*A+C11*B)/Cdet
    # Update velocities of the Lagrangian coordinates:
    theta1dot += atheta1*dt
    theta2dot += atheta2*dt
    # Update Lagrangian coordinates:
    dtheta1 = theta1dot*dt
    dtheta2 = theta2dot*dt
    theta1 += dtheta1
    theta2 += dtheta2
    
    frame1.rotate(axis=(0,0,1), angle=dtheta1)
    frame2.pos = top+frame1.axis*L1
    frame2.rotate(axis=(0,0,1), angle=dtheta2)
    
    ball2.pos=frame2.frame_to_world(ball1.pos)

    ##---------------------------------------------##

    C34 = C43 = 0.5*M2*L3*L4*cos(theta3-theta4)
    Cdet_2 = C33*C44-C34*C43
    a = .5*M2*L3*L4*sin(theta3-theta4)
    A = -(.5*M1+M2)*g*L3*sin(theta3)-a*theta4dot**2
    B = -.5*M2*g*L4*sin(theta4)+a*theta3dot**2
    atheta3 = (C44*A-C34*B)/Cdet_2
    atheta4 = (-C43*A+C33*B)/Cdet_2
    # Update velocities of the Lagrangian coordinates:
    theta3dot += atheta3*dt
    theta4dot += atheta4*dt
    # Update Lagrangian coordinates:
    dtheta3 = theta3dot*dt
    dtheta4 = theta4dot*dt
    theta3 += dtheta3
    theta4 += dtheta4
    
    frame3.rotate(axis=(0,0,1), angle=dtheta3)
    frame4.pos = top2+frame3.axis*L3
    frame4.rotate(axis=(0,0,1), angle=dtheta4)
    
    ball4.pos=frame4.frame_to_world(ball3.pos)
    

    t = t+dt
    t2 = t2+dt

## Energy check:
##    K = .5*((.25*M1+M2)*L1**2+I1)*theta1dot**2+.5*(.25*M2*L2**2+I2)*theta2dot**2+\
##        .5*M2*L1*L2*cos(theta1-theta2)*theta1dot*theta2dot
##    U = -(.5*M1+M2)*g*L1*cos(theta1)-.5*M2*g*L2*cos(theta2)
##    gK.plot(pos=(t2,K))
##    gU.plot(pos=(t2,U))
##    gE.plot(pos=(t2,K+U))
